#!/usr/bin/perl
#
#

package FXUT;
require Exporter;

@ISA = qw(Exporter);
@EXPORT_OK = qw(fxmail fxlog fxabort);

use Carp;

my $LOGFILE = "logfile";

sub fxlog
{
  my $msg = $_[0];
  my $warn = $_[1];

  open(LOG, ">> $LOGFILE") || 
    die "Failed to open log file($LOGFILE) for writing.";
  print LOG `date`;
  print LOG "$msg";
  close(LOG);
}

sub fxmail
{
  my($to, $from, $subj, $out) = @_;

  $SENDMAIL = "/usr/lib/sendmail -t -oi " ;
  open (MAIL, "| $SENDMAIL ") || &fxabort("mail(): Failed to open pipe to sendmail.");
  print MAIL "To: $to\n" ;
  print MAIL "From: $from\n";
  print MAIL "Subject: $subj\n\n" ;
  print MAIL $out;
  close(MAIL);
}

sub fxabort 
{
  my $error_msg = $_[0];

  my $msg_header = "";
  $msg_header .= "$ENV{REMOTE_HOST}:  " if defined($ENV{REMOTE_HOST});
  $msg_header .= "$ENV{REMOTE_ADDR}:  " if defined($ENV{REMOTE_ADDR});

  my $date = localtime(time);
  my $msg = $msg_header . "FATAL ERROR  $date\n";
  &fxlog($msg);

  $msg = $msg_header . $error_msg . "\n\n";
  &fxlog($msg);
  print "$msg\n";

  # if ($::db{dbhandle} ne "") {
  #   $::db->disconnect();
  #   }

  confess($error_msg);
}

sub Move
{
  my $source = $_[0];
  my $dest = $_[1];
  my $force = $_[2];
  my $nolog = $_[3];
  my $removeSourceDir = 0;
  my $source_fname = "";
  my %destinations = ();
  my @overwrites = ();
  my $file = "";

  $source =~ s/\/$//; $dest =~ s/\/$//; 
  if ($source =~ /\/([^\/]*)$/) {
    $source_fname = $1;
  } else {
    $source_fname = $source;
  }

  if (-d $source) 
  {
    $removeSourceDir = 1;
    # Check destination
    if (-d $dest) {
      if ($force eq "") {
        &fxabort("dest ($dest) exists.  Set the force flag, if you want to overwrite.");
      } else { push(@overwrites, $dest); }
    } else {
      if (-e $dest) {
        &fxabort("dest ($dest) is a file, source ($source) is a directory.");
      } else {
        &Mkdir($dest);
      }
    }

    # Get the files
    opendir(DIR, $source) || &fxabort("Can't open source dir ($source) for reading");
    my @allfiles = readdir(DIR);
    closedir(DIR);

    FILE:
    foreach $file (@allfiles)
    {
      next FILE if (($file eq ".") || ($file eq ".."));
      my $fullpath = $source . "/" . $file;
      $destinations{$fullpath} = $dest . "/" . $file;
      if (-e $destinations{$fullpath}) {
        if ($force eq "") {
          &fxabort("dest ($destinations{$fullpath}) exists.  Set the force flag, if you want to overwrite.");
        } else { push (@overwrites, $dest); }
      }
    }
  }
  elsif (-f $source)
  {
    if (-d $dest) {
      my $fullpath = $dest . "/" . $source_fname;
      $destinations{$source} = $fullpath;
      if (-e $fullpath) {
        if ($force eq "") {
          &fxabort("dest ($fullpath) exists.  Set the force flag, if you want to overwrite.");
        } else { push (@overwrites, $dest); }
      } 
    } elsif (-e $dest) {
      $destinations{$source} = $dest;
      if ($force eq "") {
        &fxabort("dest ($dest) exists.  Set the force flag, if you want to overwrite.");
      } else { push (@overwrites, $dest); }
    } else {
      $destinations{$source} = $dest;
    }
  } else {
    &fxabort("Can't move ($source), file doesn't exist.");
  }

  # log overwrites
  if (@overwrites > 0) {
    &fxlog("\t- Warning:  Overwriting " . @overwrites . " files (@overwrites).\n") if ($nolog eq "");
  }

  my @files2move = keys(%destinations);
  foreach $file (@files2move)
  {
    my $to = $destinations{$file};
    print STDERR "moving $file to $to\n" if ($debug);

    &Copy($file, $to, "nolog");
    &Remove($file, "nolog", "force");

    warn "$file renamed $to\n" if ($debug);
    &fxlog("\t- Moved $file -> $to\n") if ($nolog eq "");
  }

  &Remove($source, "force") if ($removeSourceDir == 1);
}

sub Mkdir
{
  my ($newDir, $nolog) = @_;
  $newDir =~ s/\/$//;

  if (-e $newDir) {
    &fxabort("New dir ($newDir), already exists.");
  }

  if ($newDir =~ /^(.*\/)[^\/]+$/) {
    my $newPath = $1;
    if (! -d $newPath) {
      &fxabort("Can't create new dir ($newDir) because ($newPath) doesn't exist.");
    } 
  }

  my $cmd = "mkdir $newDir";
  #print "$cmd\n";
  my $output = `$cmd`;
  &fxlog("\t- $cmd\n") if ($nolog eq "");

  if (! -d $newDir) {
    &fxabort("cmd ($cmd) failed.  Returned ($output)");
  } 
}

sub symbolic_link
{
  my $link_name = $_[0];
  my $link_to = $_[1];
  my $nolog = $_[2];

  if ($link_name eq "") {
    &fxabort("Must pass in a link name.");
  }
  if ($link_to eq "") {
    &fxabort("Must pass in a file to link to.");
  }

  if (-l $link_name) {
    &fxabort("Symbolic link $link_name already exists.");
  } elsif (-e $link_name) {
    &fxabort("$link_name already exists, and its not a symbolic link.");
  }

  if (! -e $link_to) {
    &fxabort("File to link to ($link_to) does not exist.");
  }

  my $cmd = "ln -s $link_to $link_name";
  &shellEscape($cmd);

  if (! -l $link_name) {
    &fxabort("Failed to create link to $link_to named $link_name");
  }

  if ($nolog eq "") {
    &fxlog("\t- $link_name -> $link_to\n");
  }
}

sub Remove
{
  my $file = $_[0];
  my $flags = "";

  $flags .= "f";
  $flags .= "r" if ((-d $file) && (! -l $file));
  $flags = "-" . $flags if ($flags ne "");

  if ((-e $file) || (-l $file)) {
    my $cmd = "rm $flags $file";
    my $output = `$cmd`;
    if ((-e $file) || (-l $file))  {
      &fxabort("cmd ($cmd) failed.  Returned ($output).");
    }
  } else {
    # While we might just say, the file doesn't exist, so it's removed,
    # it might be helpful to know if the file never existed
    # &fxabort("can't remove file ($file) that doesn't exist.");
  }
}

sub Touch
{
  my $newfile = $_[0];

  if (-e $newfile) {
    &fxabort("Can't touch a file ($newfile) that already exists.");
  }

  my $cmd = "touch $newfile";
  &shellEscape($cmd);

  if (! -e $newfile) {
    &fxabort("Couldn't create empty file ($newfile).");
  } else {
    &fxlog("\t- Created empty file ($newfile).\n");
  }
}

sub Copy
{
  my $source = $_[0];
  my $dest = $_[1];
  my $nolog = $_[2];
  my $flags = "";

  # parse the source file name
  $source =~ s/\/$//; $dest =~ s/\/$//; 
  if ($source =~ /\/([^\/]*)$/) {
    $source_fname = $1;
  } else {
    $source_fname = $source;
  }
  my $newfile = $dest;

  # Check source & dest file types
  if (-f $source) {
    if (-e $dest) {
      if (! -d $dest) {
        &fxlog("\t- Overwriting $dest.\n") if ($nolog eq "");
      } else {
        $newfile = $dest . "/" . $source_fname;
        if (-f $newfile) {
          &fxlog("\t- Overwriting $newfile.\n") if ($nolog eq "");
        }
      }
    }
  } elsif (-d $source) {
    $flags .= "r";
    if (-e $dest) {
      if (! -d $dest) {
        &fxabort("\t- Can not overwrite plain file ($dest) with directory ($source).");
      } else {
        $newfile = $dest . "/" . $source_fname;
        if (-d $newfile) {
          &fxlog("\t- Warning:  $source is a directory, copy of directory will named $source_fname and placed in $newfile.\n");
        } elsif (-e $newfile) {
          &fxabort("Can not overwrite plain file ($newfile) with directory ($source).");
        }
      }
    } else {
      &fxlog("\t- Notice:  $source is a directory, copy of directory will be named $dest.\n");
    }
  } else {
    &fxabort("Can't copy file ($source) that doesn't exist.");
  }

  # last sanity check
  if ($source eq $newfile) {
    &fxabort("source ($source) is the same as target ($newfile).");
  }

  # Do the copy
  $flags = "-" . $flags if ($flags ne "");
  my $cmd = "cp $flags $source $dest";
  #&fxlog("\t- $cmd\n") if ($nolog eq "");
  &shellEscape($cmd);

  # check that the file made it
  if (! -e $newfile) {
    &fxabort("Failed to copy $source -> $newfile.");
  } else {
    &fxlog("\t- Copied ($source -> $newfile).\n") if ($nolog eq "");
  }
}


sub file_datestamp
{
  # return the date a file was last modified 
  # in Informix date format

  my $datefile = $_[0];

  my %months = ('Jan' => '01', 'Feb' => '02', 'Mar' => '03',
    'Apr' => '04', 'May' => '05', 'Jun' => '06',
    'Jul' => '07', 'Aug' => '08', 'Sep' => '09',
    'Oct' => '10', 'Nov' => '11', 'Dec' => '12');

  my $lsval = _timestamp_helper($datefile);
  my @lsarray = split /\s+/, $lsval;

  my $month  = $months{$lsarray[5]};

  my $day    = $lsarray[6];
  $day = "0" . $day if ($day < 10);

  my $year   = $lsarray[7];
  # if file is less than 6 months old, time is included instead of a year
  if ($year =~ /:/) {
    my @time = localtime(time);
    $year = $time[5] + 1900;
    if ($month > $time[4]+1) {
      $year -= 1;
      }
  }

  return("$month/$day/$year");
}

sub file_datetimestamp
{
  # return the date and time a file was last modified 
  # in Informix datetime year to minute format

  my $datefile = $_[0];

  my %months = ('Jan' => '01', 'Feb' => '02', 'Mar' => '03',
    'Apr' => '04', 'May' => '05', 'Jun' => '06',
    'Jul' => '07', 'Aug' => '08', 'Sep' => '09',
    'Oct' => '10', 'Nov' => '11', 'Dec' => '12');

  my $lsval = _timestamp_helper($datefile);
  my @lsarray = split /\s+/, $lsval;
  my $month  = $months{$lsarray[5]};

  my $day    = $lsarray[6];
  $day = "0" . $day if ($day < 10);

  my $year   = $lsarray[7];
  # if file is less than 6 months old, time is included instead of a year
  if ($year =~ /:/) {
    my @time = localtime(time);
    $year = $time[5] + 1900;
    if ($month > $time[4]+1) {
      $year -= 1;
      }
  }

  my $time = $lsarray[7];
  # if a year is included instead of a time, just use midnight...
  if ($time !~ /^\d+:\d+$/) {
    $time = "00:00";
  }

  return("$year-$month-$day $time");
}

sub _timestamp_helper
{
  my $file = $_[0];
  my $lsval;

  # get an ls -l of the file/dir
  if (-f $file) {
    $lsval = `ls -l $file`;
  } elsif (-d $file) {
    $lsval = `ls -ld $file`;
  } else {
    &fxabort("($file) is not a file or a directory!");
  }

  &fxabort("Failed to get ls of ($file)") if ($lsval eq "");

  return($lsval);
}


sub shellEscape
{
  my $cmd = $_[0];
  
  # try to execute it
  my $output = system($cmd);
  
  my $status = ($? >> 8);
  if ($status != 0) {
    &fxabort("'$cmd' failed w/ status ($status)\n\tReturned:  ($output)");
  }

  return($output);
}

sub showenv_html
{
    my @env = (DOCUMENT_ROOT, REMOTE_ADDR, REMOTE_HOST, 
               HTTP_REFERER, SERVER_NAME, REQUEST_URI, HTTP_HOST);
    
    print "<BODY>\n";
    foreach my $env_var (@env) {
        print "<BR><FONT COLOR=red>$env_var</FONT> =====> <FONT COLOR=blue>$ENV{$env_var}</FONT>\n";
        }
    print "<BR><BR>\n";
    foreach my $env_var (keys %ENV) {
        print "<BR><FONT COLOR=red>$env_var</FONT> =====> <FONT COLOR=blue>$ENV{$env_var}</FONT>\n";
        }
    print "</BODY>\n";



}
