/*
 * Include file: fxsel.h
 * Description:  Include file for the SQL SELECT statement.
 *		 Contains defines, macros, the SELECT tree structure
 *		 and external references.
 */
#include <fxsql.h>
#include <valfuncs.h>

#define SELLIST    0

/* Expression factor types
 */
#define CONSTANT   1			/* constant */
#define VARIABLE   2			/* variable */
#define SGLFIELD   3			/* single field */
#define AGGREGATE  4			/* aggregate function */
#define DATEFUNC   5			/* date function */
#define NESTEXPR   6			/* nested expression */

/* Optional SELECT clauses - main SELECT as well as subqueries
 */
#undef  WHERE
#define WHERE	   1
#define GROUP	   2
#define HAVING	   3
#define ORDER	   4
#define INTO	   5

/* Boolean operators
 */
#undef	LT
#define EQ	   1
#define NE	   2
#define LT	   3
#define GT	   4
#define LE	   5
#define GE	   6
#define IN	   7
#define BETWEEN	   8
#define MATCHES	   9
#define LIKE	  10
#define ISNULL	  11

/* Subquery array operators
 */
#define EXISTS	   1
#define ANY	   2
#define ALL	   3

/* Date functions
 */
#define DAY	   1
#define MONTH	   2
#define	YEAR	   3
#define WEEKDAY    4
#define MDY	   5

/* Aggregate functions:
*/
#undef  MAX
#undef  MIN
#define COUNT	   1
#define COUNTA	   2		/* count(*) */
#define MAX	   3
#define MIN	   4
#define SUM	   5
#define AVG	   6

#define ALSNAMSIZ   18
#define INSTBLALIAS "_____xXxINS"
#define NETIFILLFLD "__fIlLtOlEn4"

#define isidchar(c)	(isalnum( c) || c == '_')
#define umalloc(x)	malloc((unsigned) x + 20)

/* The SELECT tree structure
 */
typedef struct {
	char	 slfFnam[FLDNAMSIZ+1];	/* field name */
	char *	 slfTbl;		/* table */
	DBFIELD *slfFptr;		/* pointer to table field structure */
	short	 slfPfinal;		/* final pointer to table field? -
					      may change with temporary table */
	short	 slfStart;		/* substring start */
	short	 slfEnd;		/* substring end */
	short	 slfIsrowid;	 	/* is the field ROWID? */
} SFIELD;

typedef struct _dtexpr {
	VALUE	sdtVal;			/* value and type */
	SFIELD	sdtFld;			/* table field */
	char	sdtAddop;		/* operator: '+' or '-' */
	struct _dtexpr *sdtRexp;	/* right date expression operand */
} DTEXPR;

typedef struct {
	short	sdfType;		/* DAY, MONTH, YEAR, WEEKDAY */
	DTEXPR	sdfDtexp;		/* argument to date functon */
} DTFUNC;

typedef struct {
	VALUE	sfcVal;			/* value and type */
	short	sfcSign;		/* 1 = positive, -1 = negative */
	short	sfcEtype;		/* expression type: CONSTANT,	
					      VARIABLE, SGLFIELD,
					      AGGREGATE, DATEFUNC */
	DTFUNC *sfcDtfunc;		/* date function */
	short	sfcAggreg;		/* aggregate type: COUNT, COUNTA,
					      MAX, MIN, SUM, AVG */
	long    sfcNonullcount;		/* count of applicable records for
					      AVG aggregate (NULLs ignored) */
	SFIELD	sfcFld;			/* field description */
	struct _expr *	sfcExp;		/* nested expression */
} FACTOR;

typedef struct _term {
	FACTOR	stmFact;		/* left factor operand */
	char	stmMultop;		/* operator: '*' or '/' */
	struct _term *	stmRtrm;	/* right term operand */
} TERM;

typedef struct _expr {
	short	 sxpType;		/* data type, in fxtypes.h */
	short	 sxpSiz;		/* data size */
	short	 sxpOffs;		/* beginning byte within substring,
					      or number of bytes right of
					      decimal for decimal type */
	short	 sxpIsagg;		/* aggregate function within exp. */
	TERM	 sxpTerm;		/* left term operand */
	char	 sxpAddop;		/* operator: '+' or '-' */
	struct _expr *	sxpRexp;	/* right expression operand */
	char	 sxpFldnm[FLDNAMSIZ+1];	/* field name if only one field in
					      expression, "2" if more than
					      one field */
	char	 sxpLabel[FLDNAMSIZ+1];	/* select list label */
	short	 sxpSort;		/* label in ORDER BY list? */
	short	 sxpAutlbl;		/* label auto-generated? */
	char *	 sxpAlltbl;		/* table.* in the select list */
	struct _expr *	 sxpNexp;	/* next expression of select list */
	short	 sxpLendisp;		/* field display widths */
	short	 sxpX;			/* input line offset of exp. text */
	short	 sxpY;			/* input line number of exp. text */
} EXPR;

typedef struct _bracket {
	VALUE	sbkBegval;		/* beginning value of bracketed range */
	short	sbkBincl;		/* range inclusive of beginning value */
	VALUE	sbkEndval;		/* ending value of bracketed range */
	short	sbkEincl;		/* range inclusive of ending value */
	struct _bracket *sbkNext;	/* next bracketed range */
} VALBRACK;

typedef struct _listv {
	VALUE	slvVal;			/* value and type */
	short	slvTo;			/* TO range operator */
	struct _listv *	slvNxt;		/* next value in list */
} LISTV;

typedef struct _optiminfo {
	SFIELD *  sopFld;		/* pointer to field of WHERE clause */
	short	  sopCount;		/* number of times field in factor */
	short     sopIdxok;		/* field can be used in index */
	VALBRACK *sopBracket;		/* range of possible constant values */
	short	  sopBrkidx;		/* index for bracket? */
	struct _optiminfo *sopNext;	/* optimization info for next field */
} OPTIMINFO;

typedef struct {
	short	sbfFlag;		/* 0=false, 1=true, -2=undefined */
	short	sbfNot;			/* NOT operator */
	EXPR	sbfLexp;		/* left expression */
	short	sbfBop;			/* boolean comparator */
	EXPR *	sbfRexp;		/* right expression */
	short	sbfQop;			/* subquery operator:
					      EXISTS, ANY, ALL */
	struct _select *sbfSlct;	/* nested select */
	LISTV *	sbfVlst;		/* value list */
	struct _cond *	sbfCond;	/* nested condition */
	OPTIMINFO    *	sbfOptim;	/* table access optimization info */
} BFACTOR;

typedef struct {
	BFACTOR * sdxBfact;		/* boolean factor with join fields */
	VALBRACK *sdxCurbrack;		/* current bracket of range */
} SELINDEX;

typedef struct _selflds {
	SFIELD *ssfFld;			/* pointer to field structure */
	short	ssfInlist;		/* in-selection-list flag */
	short	ssfInwhere;		/* in-where-clause flag */
	struct _selflds *ssfNfld;	/* pointer to next field of FROM tbl */
} SELFLDS;

typedef struct _frmtbl {
	char	 sftTnam[TBLNAMSIZ+1];	/* table name */
	char	 sftAlias[ALSNAMSIZ+1]; /* table alias */
	DBHEAD * sftTptr;		/* pointer to table header structure */
	long	 sftRecnum;		/* current record number */
	long	 sftSavnum;		/* saved record number */
	DBINDEX *sftIndex;		/* pointer to DBINDEX structure */
	short	 sftRevidx;		/* access in reverse index order? */
	short	 sftByrowid;		/* access by ROWID? */
	short    sftOutergrp;		/* outer join group */
	short    sftOutparent;		/* parent outer join group */
	short	 sftOuterjoin;		/* is table OUTER joined? */
	struct _frmtbl *sftPrimeout;	/* primary table of OUTER join group */
	short    sftOutnull;		/* NULL result for OUTER join group? */
	SELINDEX sftIdxrel[NPARTS];	/* relations of index parts */
	BFACTOR *sftSavfct[NPARTS];	/* save boolean factors of idx parts */
	struct _frmtbl *sftNtbl;	/* next table of FROM table list */
	SELFLDS *sftSflds;		/* list of selected fields */
	short	 sftFlag;		/* optimization flag:
					      inititally set to 1 if table used;
					      then set to access order number */
	char *	 sftTmpt;		/* temporary table to provide
					      alternate index */
	char *	 sftXbuf;		/* alternate buffer to save previous
					      record read for evaluating
					      group breaks */
	short	 sftX;			/* input line offset of table text */
	short	 sftY;			/* input line number of table text */
} FRMTBL;

typedef struct _fldlist {
	SFIELD	sflFld;			/* field name */
	short	sflFlag;		/* sort order (for order-by list):
						   1 ascending, -1 descending;
					   in select list (for group-by list) */
	short	sflInidx;		/* in index */
	struct _fldlist *sflNxt;	/* next field of field list */
	struct _fldlist *sflAltlist;	/* any alternative flds for ORDER BY */
	struct _fldlist *sflCurfld;	/* the current of the
					   primary/alternative fields */
	short	sflX;			/* offset into input line of fld name */
	short	sflY;			/* input line number of field name */
} FLDLIST;

/* Structures used to determine access order of FROM tables
 *    based on join indices.
 */
typedef struct _bfctlst {
	BFACTOR * sfsFact;		/* boolean factor in a list */
	struct _bfctlst *sfsNext;	/* next item of the factor list */
} BFACTLST;
typedef struct _ordfld {
	DBFIELD * sofFptr;		/* database field pointer */
	short	  sofSortord;		/* sort order: 1 ascending,
						      -1 descending */
	short	  sofReqidx;		/* index required for ORDER BY */
	short	  sofIdxcand;		/* candidate for ORDER BY index */
	BFACTLST *sofFldfacts;		/* boolean factor list for each field */
	BFACTLST *sofCurfact;		/* current item of the factor list */
	struct _ordfld *sofCurfld;	/* current item of ORDER BY fld. list */
	struct _ordfld *sofNext;	/* next item of ORDER BY field list */
} ORDFLD;
typedef struct _joinlst {
	ORDFLD *  sjlOrdfld;		/* pointer to ORDER BY field struct. */
	struct _joinlst *sjlNext;	/* next item of join field list */
} JOINLIST;
typedef struct {
	FRMTBL *  stoFrmtbl;		/* pointer to FRMTBL structure */
	ORDFLD    stoOrdflds[NPARTS];	/* fields of ORDER BY clause */
	BFACTOR * stoEqfacts[NPARTS];	/* boolean factors of fields equal to
					      constants, fields to be merged
					      with ORDER BY fields */
	JOINLIST  stoJoin[NPARTS];	/* list of ORDER BY field joins for each
					      field */
	short	  stoNumflds;		/* number of index field candidates */
	short	  stoNumeqfacts;	/* number of constant equality factors*/
	short	  stoPhase;		/* index evaluation phase */
} TBLORD;

typedef struct _bterm {
	BFACTOR	*sbtFact;		/* left boolean factor */
					/* AND */
	struct _bterm *	sbtRtrm;	/* right boolean term */
	short	 sbtObjoin;		/* candidate for ORDER BY join */
} BTERM;

typedef struct _cond {
	short	scnAlways;		/* 0=always false, 1=always true
					       -1=undefined */
	BTERM	scnTerm;		/* left boolean term */
					/* OR */
	struct _cond *	scnRcond;	/* right condition */
} COND;

typedef struct _select {
	short	 selNum;	  /* SELECT structure number */
	short	 selUniq;	  /* no duplicate results */
	short	 selAll;	  /* SELECT * */
	EXPR	 selExp;	  /* initial expression of select list tuple */
	FRMTBL * selFtp;	  /* FROM table list */
	short	 selNtbls;	  /* number of tables in FROM table list;
					0 = not yet ordered */
	COND *	 selWcond;	  /* where condition */
	FLDLIST *selGlst;	  /* fields to group by */
	COND *	 selHcond;	  /* having condition */
	struct _select *selOuter; /* outer select, if this is a nested select */
	DBHEAD * selTmptp;	  /* temporary table DBHEAD pointer */
	VALUE   *selPrvval;	  /* if nested select, previous index value */
	short	 selResprv;	  /* if nested select, previous result (0/1) */
	short	 selFmstartmd;    /* mode argument for fmstart() */
	short    selFirstreadflg; /* first read flag */
	short	 selMaxoutgrp;	  /* maximum OUTER join group number
					0 = none */
	short	 selBrkidx;	  /* index for bracket? */
} SELECT;

/* Fields assigned to by UPDATE SET clause.
 */
typedef struct _updfield {
	char	upfFldnam[FLDNAMSIZ+1]; /* field name */
	struct _updfield *upfNxtfld;	/* next field */
	short	upfX;			/* input line offset of field name */
	short	upfY;			/* input line number of field name */
} UPDFIELD;

#define SBFNULL	(BFACTOR *)0
#define SBKNULL	(VALBRACK *)0
#define SBTNULL	(BTERM *)0
#define SCNNULL	(COND *)0
#define SDFNULL	(DTFUNC *)0
#define SDTNULL	(DTEXPR *)0
#define SELNULL	(SELECT *)0
#define SFCNULL	(FACTOR *)0
#define SFLNULL	(FLDLIST *)0
#define SFSNULL	(BFACTLST *)0
#define SFTNULL	(FRMTBL *)0
#define SJLNULL	(JOINLIST *)0
#define SLFNULL	(SFIELD *)0
#define SLVNULL	(LISTV *)0
#define SOFNULL	(ORDFLD *)0
#define SOPNULL	(OPTIMINFO *)0
#define SSFNULL	(SELFLDS *)0
#define STMNULL	(TERM *)0
#define STONULL	(TBLORD *)0
#define SXPNULL	(EXPR *)0
#define UPFNULL	(UPDFIELD *)0

/* Is the SELECT expression just 1 field, with no arithmetic or function
 *     or substring starting at a character other than the first?
 */
#define is1fldexp(e)	((e)->sxpRexp == SXPNULL && \
			 (e)->sxpTerm.stmRtrm == STMNULL && \
			 (e)->sxpTerm.stmFact.sfcEtype == SGLFIELD && \
			 ((e)->sxpTerm.stmFact.sfcVal.vlType != CHARTYPE || \
			  (e)->sxpTerm.stmFact.sfcFld.slfStart <= 1))
#define getexpfld(e)	(&(e)->sxpTerm.stmFact.sfcFld)

extern short selmode;
extern short selcnt;
extern SELECT sglSelect;
extern FLDLIST sortfield;
extern UPDFIELD updfld;
extern short phase;
extern int flfd;

typedef struct _alloclist {
	struct _alloclist *salPrev;
	char *	salMem;
} ALLOCLIST;
#define ALNULL	(ALLOCLIST *)0
extern ALLOCLIST *lastalloc;

extern void free();
extern long atol();
extern double atof();

extern SELECT *selparser();
extern SELECT *typecheck();
extern short sel_clause();
extern short comp_op();
extern short agg_spec();
extern LISTV *const_list_expr();
extern FRMTBL *lkupfromtbl();
extern FRMTBL *lkup_ffld();
extern short iskeyword();
extern short is_char();
extern short get_id();
extern char *parsalloc();
extern COND *chknest_cond();
extern char *loadsymbol();
extern char *rwgets();
extern char *strtrim();
extern DBHEAD *tballoc();
extern char *mkdskname();
