/*
-----------------------------------------------------------------------------
multi-thread handling

this feature owes it's existance to Callum Gibson in australia because
without his expertise and advice it would never have been completed.
-----------------------------------------------------------------------------

 +--------------------------------------------------------------------------+
 | Worldwide Copyright (c) 1987-2012 Byte Designs Ltd. All rights reserved. |
 |                                                                          |
 |                            20568 - 32 Avenue                             |
 |                               LANGLEY, BC                                |
 |                             V2Z 2C8 CANADA                               |
 |                                                                          |
 |                       Sales: sales@bytedesigns.com                       |
 |                     Support: support@bytedesigns.com                     |
 |              Phone: (604) 534 0722     Fax: (604) 534 2601               |
 +--------------------------------------------------------------------------+
                                Version 7.0
 
-----------------------------------------------------------------------------
*/

#if( ISTHREADED == 0 )              /* not threaded */
#define ISMTINIT(I) ;;
#define ISMTBEG(I)  ;;
#define ISMTEND(I)  ;;
#define ISMTDROP(I) ;;
#define ISMTWAIT(I1,I2)	break;
#define ISMTSIG(I)  ;;
#endif


#if( ISTHREADED == 1 )              /* SunOS */
#include <lwp/lwp.h>
#define ISMTLOCK     mon_t
#define ISMTINIT(MX) mon_create( &(MX) )
#define ISMTBEG(MX)  mon_enter( MX )
#define ISMTEND(MX)  mon_exit( MX )
#define ISMTDROP(MX) mon_destroy( MX )
#define ISMTWAIT(I1,I2)	break;
#define ISMTSIG(I)	break;
#endif


#if( ISTHREADED == 2 )              /* Solaris */
#include <synch.h>
#define ISMTLOCK     mutex_t
#define ISMTCOND     cond_t
#define ISMTINIT(MX) mutex_init( &(MX), USYNC_THREAD, NULL )
#define ISMTBEG(MX)  mutex_lock( &(MX) )
#define ISMTEND(MX)  mutex_unlock( &(MX) )
#define ISMTDROP(MX) mutex_destroy( &(MX) )
#define ISMTWAIT(C, MX)  cond_wait( &(C), &(MX) )
#define ISMTSIG(C)   cond_signal( &(C) )
#endif


#if( ISTHREADED == 5 )              /* pthreads 0.9.2 */
#include <pthread.h>
#define ISMTLOCK     pthread_mutex_t
#define ISMTCOND     pthread_cond_t
#ifdef _INCLUDE_HPUX_SOURCE
# define ISMTINIT(MX) pthread_mutex_init( &(MX), pthread_mutexattr_default )
#else
# define ISMTINIT(MX) pthread_mutex_init( &(MX), NULL )
#endif
#define ISMTBEG(MX)  pthread_mutex_lock( &(MX) )
#define ISMTEND(MX)  pthread_mutex_unlock( &(MX) )
#define ISMTDROP(MX) pthread_mutex_destroy( &(MX) )
#define ISMTWAIT(C, MX)  pthread_cond_wait( &(C), &(MX) )
#define ISMTSIG(C)   pthread_cond_signal( &(C) )
#define ISCDINIT(C)  pthread_cond_init( &(C), NULL )
#define ISCDDROP(C)  pthread_cond_destroy( &(C) )
#endif

