/*
 * Include file: valfuncs.h
 * Description:  The value functions are used to manipulate data values of
 *		 all data types, independent of the implementation of the
 *		 decimal and money types in a particular port.
 *		 Currently all numerics are stored programmatically
 *		 (not in the database) in decimal structures (dec_t).
 */

#ifdef UNIX 
#   ifdef VAXVMS
#       include <limits.h>
#       include <float.h>

#       define MAXSHORT   SHRT_MAX
#       define MINSHORT   SHRT_MIN
#       define MAXLONG    LONG_MAX
#       define MINLONG    LONG_MIN
#       define MAXDOUBLE  DBL_MAX
#       define MAXFLOAT   FLT_MAX
#   else
#       ifdef NOVALUEH
#           include "fxvalue.h"
#       else 
#           include <values.h>
#       endif
#       undef  MINDOUBLE
#       undef  MINFLOAT
#   endif
#endif

#ifndef MSDOS
#   ifdef NOVALUEH
#       include "fxvalue.h"
#   else 
#       include <values.h>
#   endif
#   undef  MINDOUBLE
#   undef  MINFLOAT
#   undef  MINLONG
#   undef  MINSHORT
#   define MINLONG    (-MAXLONG)-1
#   define MINSHORT   (-MAXSHORT)-1
#else
#   include <limits.h>
#   include <float.h>

#   define MAXSHORT   SHRT_MAX
#   define MINSHORT   SHRT_MIN
#   define MAXLONG    LONG_MAX
#   define MINLONG    LONG_MIN
#   define MAXDOUBLE  DBL_MAX
#   define MAXFLOAT   FLT_MAX
#endif
#define MINDOUBLE  -MAXDOUBLE
#define MINFLOAT   -MAXFLOAT


#define UNDEF	  -1

#ifdef NO_DEC
#define	valcvdbl(n,v)	   (v)->vlNval = (double) (n)
#else
#define	valcvdbl(n,v)	   deccvdbl( (double) (n), &(v)->vlNval)
#endif

#define valcvint	valcvdbl
#define valcvlong	valcvdbl
#define valcvflt	valcvdbl

#ifdef DEC_DEC
#define valadd(v1,v2,res)  decadd( &(v1)->vlNval, &(v2)->vlNval, &(res)->vlNval)
#define valsub(v1,v2,res)  decsub( &(v1)->vlNval, &(v2)->vlNval, &(res)->vlNval)
#define valmul(v1,v2,res)  decmul( &(v1)->vlNval, &(v2)->vlNval, &(res)->vlNval)
#define valdiv(v1,v2,res)  decdiv( &(v1)->vlNval, &(v2)->vlNval, &(res)->vlNval)
#else
#define valadd(v1,v2,res)  valarith(v1,'+',v2,res)
#define valsub(v1,v2,res)  valarith(v1,'-',v2,res)
#define valmul(v1,v2,res)  valarith(v1,'*',v2,res)
#define valdiv(v1,v2,res)  valarith(v1,'/',v2,res)
#endif

typedef struct {
	char *	vlSval;			/* pointer to string value */
	short   vlSlen;			/* maximum length of string */
#ifdef NO_DEC
	double  vlNval;			/* numbers stored in doubles */
#else
	dec_t	vlNval;			/* numbers stored in dec_t */
#endif
	short	vlType;			/* data type, in fxtypes.h */
	short	vlNull;			/* is value null? */
} VALUE;

typedef struct {
	char    limChar[2];
#ifdef NO_DEC
	double  limDouble;
#else
	dec_t	limDec;
	dec_t	limSmflt;
	dec_t	limLong;
	dec_t	limShort;
#endif
} LIMITS;

#define VALNULL	(VALUE *)0

extern VALUE maxval;
extern VALUE minval;

char *val_getp();
long valtolong();
float valtoflt();
double valtodbl();
