/************************************************************************
*                       - INFOFLEX System -
* This source code is CONFIDENTIAL and PROPRIETARY to CUSTOMWARE COMPUTING,
* INC. (CWC). The information contained herein may not be used, copied or
* disclosed in whole or in part except as permitted by written agreement
* signed by an officer of CWC.
*
* COPYRIGHT (c) 1986 CUSTOMWARE COMPUTING, INC., BURLINGAME, CALIFORNIA.
* ALL RIGHTS RESERVED.
*
* Version: %V%  %R%
* Date: %D%  %T%
*************************************************************************/

/*F
 *    Name: termlib.c  - Library of general routines for termflex.
 *
 *    Class: F
 *
 *    Author:  Gerard Menicucci
 *
 *    Description:
 *
 *    Returns:
 *
 *    Warnings:
 *
 */

#include "termflex.h"

/*****************************************************************

      G E N E R I C     C O D E

*******************************************************************/

printseq(  ptermtbl, ptermseq, maxseq )
TERMTBL *ptermtbl;
char **ptermseq;
int maxseq;
{
    int i, j;
    int maxsize;

   printf(" -----------------------------------------------------\n");
   printf("\n\n");
   printf( "Tag Name   Description                 Sequence  \n");
   printf( "---------- --------------------------- ----------------  \n");

   for (i = 0; i < maxseq; ++i, ++ptermtbl) {

      printf( "%-10.10s %-30.30s ",
               ptermtbl->tm_tagname,
               ptermtbl->tm_prompt);

      maxsize = strlen( ptermseq[i]);
      for (j = 0; j < maxsize; ++j)
          printf( "%4d", ptermseq[i][j] );
      printf( "\n");
      }

}


readseq( fp, ptermtbl, ptermseq, maxseq, maxsize)
FILE *fp;
TERMTBL *ptermtbl;
char **ptermseq;
int maxseq;
int maxsize;
{
     char *ptr;
     char line[120];
     int linectr;
     int codectr;
     char word[95];
     char charcode;
     int intcode;
     int error = NO;
     int index;



     linectr = 0;
     for (;;) {
         ptr = termgetline( line, sizeof(line), fp);
         if ( ptr == NULL)  /* end of file condition */
             break;
         /*
         printf("fgets: %s", line); fflush( stdout);
         */

         /* skip over white space */
         for (; *ptr == ' ' || *ptr == '\t' ; ++ptr);
         if (*ptr == '\0' || *ptr == '@') /* if newline then skip line  */
             continue;

         if (strncmp( "end", ptr, 3) == 0)
            break;

         /* We have reached a line containing control codes */
/*
         printf( "%3d.   ", linectr); fflush( stdout);
*/
         for (codectr = 0; ; ++codectr) {

             ptr = getword( word, ptr);

             if ( word[0] == '\0')
                break;

             if ( word[0] == ';' || word[0] == ':')
                break;

             if (linectr >= maxseq)  {
                 error = YES;
                 printf( "Number Sequences  exceeds maximum");
                 fflush( stdout);
                 break;
                 }

             if (codectr >= maxsize) {
                 error = YES;
                 printf(
                   "Line# %d contains too many codes", linectr);
                 fflush( stdout);
                 break;
                 }

             if ( codectr == 0) {
                for (index = 0; index < maxseq &&
                      strncmp( ptermtbl[index].tm_tagname, word, 8) != 0; ++index);
                if ( index >= maxseq) {
                   error = YES;
                   printf( "Tag invalid - %s\n", word);
                   fflush( stdout);
                   break;
                   }

/*
                printf( "%-12.12s", word); fflush( stdout);
*/
                continue;
                }

             intcode = atoi( word);
             charcode =  intcode;
             if (0 > intcode){
                    error = YES;
                    printf( "Line# %d, Code# %d is not numeric",
                           linectr, codectr);
                    fflush( stdout);
                    break;
                    }

/*
            printf( "%4d", code); fflush( stdout);
*/
            ptermseq[ index][ codectr - 1] = charcode;


          }  /*   end of inner for loop  */

          linectr++;

/*
          printf( "\r\n"); fflush( stdout);
*/

         }   /* end of outer for loop  */

   if ( error == YES) {
        printf("\r\n\r\nPress RETURN key to continue...");
        termgetline(  line, sizeof line, stdin);
        printf("\r\n\r\n\r\n");
        }

}




writeseq( fp, ptermtbl, ptermseq, maxseq, maxsize)
FILE *fp;
TERMTBL *ptermtbl;
char **ptermseq;
int maxseq;
int maxsize;
{
   int i, j;
   int cc;

   fprintf( fp,
"@Tag name  Sequence                         Description\n");
   fprintf( fp,
"@--------  -------------------------------- -----------------------------\n");

   for (i = 0; i < maxseq; ++i, ++ptermtbl) {

      fprintf(fp,  "%-10.10s ", ptermtbl->tm_tagname);

      maxsize = strlen( ptermseq[i]);
      for (j = 0; j < maxsize; ++j) {
          cc = (ptermseq[i][j] & 0xff);
          fprintf( fp, "%4d", cc);
          }
      /* pad with blanks  */
      for (; j < 8 ; ++j)
          fprintf( fp, "%.4s", "     ");

      fprintf( fp, "; %-30.30s ", ptermtbl->tm_prompt);

      fprintf( fp, "\n");
      }


   fprintf( fp, "\nend\n\n");

}


char *getword( w, p)
char *w;
char *p;
{
/*
     printf( "getword: before"); fflush( stdout);
*/
     /* skip white space  && delimiters only if in first position  */
     for (; *p == ' ' || *p == '\t'; )
            *p++;

     /* copy buffer to word variable */
     while( *p != ' ' && *p != '\t' && *p != '\0' && *p != ';' && *p != ':') {
          *w++ = *p++;
          }
     *w = '\0';
/*
     printf( "getword: after"); fflush( stdout);
*/

     return( p);
}

char *getseq( w, p)
char *w;
char *p;
{

     /* copy buffer to word variable */
     while( *p != '\0' && *p != ':') {
          *w++ = *p++;
          }

     *w = '\0';

     return( p);
}






char    *termgetline(s, n, iop)
char    *s;
int    n;
register FILE *iop;
{
    register int    c;
    register char    *cs;

    cs = s;
    while (--n > 0 && (c = getc(iop)) != EOF && c != '\n')
        *cs++ = c;
    *cs = '\0';
    return ((c == EOF && cs == s) ? NULL : s);
}


loadtbl( filename, ptermtbl, n_termtbl)
char *filename;
TERMTBL *ptermtbl;
int *n_termtbl;
{
    FILE     *fp;
    char line[120];
    int    i, j;
        char *p1;
        char *p2;
        extern char *strcompress();
        extern char *strchr();

    if ((fp = fopen(filename, "r")) == NULL) {
            printf("Can't open %s\n", filename);
            return -1;
        }

    for (i = 0; termgetline(line, sizeof(line), fp) != 0
                    && i < *n_termtbl; ++i, ++ptermtbl)  {

             p2 = line;
             for ( j = 0 ; p2 != NULL ; ++j) {

                 p1 = p2;
                 if ( NULL != (p2 = strchr( p1, ':')) ) {
                     *p2 = '\0';
                     ++p2;
                      }
/*
                 printf( "\n\rword = %s", p1);
*/
                 if ( j == 0)  {
                    strcompress( p1);
                    strncpy( ptermtbl->tm_prompt, p1, 49);
                    }
                 else if ( j == 1)  {
                    strcompress( p1);
                    strncpy( ptermtbl->tm_tagname, p1, 14);
                    }
                 else if ( j == 2) {
                    strcompress( p1);
                    strncpy( ptermtbl->tm_termcap, p1, 14);
                    }
/*
                printf( "\n\r%d - %d) prompt=%s: tag=%s: termcap=%s:",
                    i, j, ptermtbl->tm_prompt, ptermtbl->tm_tagname,
                    ptermtbl->tm_termcap);
*/

                 }  /* end of inner loop */

             }  /* end of outer loop  */

        *n_termtbl = i;


    fclose(fp);

        return( 0);
}






x_shtobuf(buf, i)
char *buf;    /* buffer where machine-independent short should be placed */
int i;        /* integer holding a 16-bit short in its low 16 bits */
{
    buf[0] = (i & CMASK);        /* mask out all but low 8 bits */
    buf[1] = ((i >> 8) & CMASK);
    return(0);
}




int x_buftosh(buf)
char *buf;    /* buffer where short is stored in machine-independent format */
{
    short i;

    i = ((buf[0] & CMASK) | ((buf[1] & CMASK) << 8));
    return (int) i;
}


writecap( fp, ptermtbl, ptermseq, maxseq, maxsize)
FILE *fp;
TERMTBL *ptermtbl;
char **ptermseq;
int maxseq;
int maxsize;
{
   int i, j;
   int cc;
   int ctl_flag;
   char capstr[150];
   char charstr[150];

   fprintf( fp,
"@Tag name  Sequence                         Description\n");
   fprintf( fp,
"@--------  -------------------------------- -----------------------------\n");

   for (i = 0; i < maxseq; ++i, ++ptermtbl) {

      
      fprintf(fp,  "%-10.10s ", ptermtbl->tm_tagname);

      maxsize = strlen( ptermseq[i]);
      capstr[0] = '\0';
      for (j = 0; j < maxsize; ++j, strcat( capstr, charstr)) {

          cc = ptermseq[i][j] & 0xff;
          
          switch( cc) {
             case 8:
                strcpy( charstr, "\\b");
                break;
             case 9:
                strcpy( charstr, "\\t");
                break;
             case 10:
                strcpy( charstr, "\\n");
                break;
             case 12:
                strcpy( charstr, "\\f");
                break;
             case 13:
                strcpy( charstr, "\\r");
                break;
             case 27:
                strcpy( charstr, "\\E");
                break;
             case ' ':
                strcpy( charstr, " ");
                break;
             case '!': /* ^ */
                strcpy( charstr, "\\041");
                break;
             case ':': /* ^ */
                strcpy( charstr, "\\072");
                break;
            default:
                if ( cc >= 0 && cc <= 31) {
                   charstr[0] = '^';
                   charstr[1] = cc + 64;
                   charstr[2] = '\0';
                   continue;
                   }
                if ( cc >= 127) {
                   sprintf( charstr, "\\%o", cc);
                   continue;
                   }
                charstr[0] = cc;
                charstr[1] = '\0';
            } /* end of switch */

          }

      strcat( capstr, ":");

      fprintf( fp, "%-30s", capstr);

      fprintf( fp, " %-30s ", ptermtbl->tm_prompt);

      fprintf( fp, "\n");
      }


   fprintf( fp, "\nend\n\n");

}



readcap( fp, ptermtbl, ptermseq, maxseq, maxsize)
FILE *fp;
TERMTBL *ptermtbl;
char **ptermseq;
int maxseq;
int maxsize;
{
     int cc;
     int i;
     char *ptr;
     char line[200];
     int linectr;
     int capctr;
     int caplength;
     char word[150];
     int error = NO;
     int index;
     int capindex, seqindex;
     char octalstr[10];
     char capstr[150];
     char charcc;
     unsigned int ucc;

     capctr = 0;
     for (linectr = 0; ;) {

         ++linectr;

         ptr = termgetline( line, sizeof(line), fp);
         if ( ptr == NULL)  /* end of file condition */
             break;

         /* printf("fgets: %d) %s", linectr, line); fflush( stdout); */

         /* skip over white space */
         for (; *ptr == ' ' || *ptr == '\t' ; ++ptr);
         if (*ptr == '\0' || *ptr == '@') 
             continue;

         if (strncmp( "end", ptr, 3) == 0)
            break;

         /* We have reached a line containing control codes */

         capctr++;

         if (capctr > maxseq)  {
             error = YES;
             printf( "Number Sequences  exceeds maximum");
             fflush( stdout);
             goto end;
             }

          ptr = getword( word, ptr);
          for (index = 0; index < maxseq &&
                strncmp( ptermtbl[index].tm_tagname, word, 8) != 0; ++index);
          if ( index >= maxseq) {
             error = YES;
             printf( "Tag invalid - |%s| line=%d\n", word, linectr);
             fflush( stdout);
             goto end;
             }


         /* get capstr */
         /* skip over white space */
         for (; *ptr == ' ' || *ptr == '\t' ; ++ptr);
         ptr = getseq( capstr, ptr);
         if ( *ptr != ':') {
             printf( "Expected ':' to end cap string - line=%d", linectr);
             fflush( stdout);
             error = YES;
             goto end;
             }

        /* printf("\ncapstr=|%s|", capstr); */

       /* convert cap string to decimal format */
       capindex = 0;
       seqindex = 0;
       caplength = strlen( capstr);
       for (; capindex < caplength;) {

         if (seqindex + 1 >= maxsize) {
             error = YES;
             printf( "Line# %d contains too many codes", linectr);
             fflush( stdout);
             goto end;
             }

          cc = capstr[ capindex++] & 0xff; 
         /*  printf("capstr=%s, %c, %d", capstr, capstr[capindex], cc); */
           /* printf("capstr=%s, %c\n", capstr, cc);  */

          if ( cc == '\\' ) {

              if ( capindex >= caplength )  /* end of string */
                goto setterm;         

              cc = capstr[ capindex++] & 0xff;

              switch( cc) { 
                case 'b':
                   cc = 8;
                   break;
                case 't':
                   cc = 9;
                   break;
                case 'n':
                   cc = 10;
                   break;
                case 'f':
                   cc = 12;
                   break;
                case 'r':
                   cc = 13;
                   break;
                case 'E':
                   cc = 27;
                   break;
                default:

                  if (cc >= 48 && cc <= 57) {
                     /* octal number - must be at least 2 more digits */
                     if ( capindex + 1 >= caplength ) {
                        printf( "Unexpected end of cap string for octal (%d)",
                              linectr);
                        fflush( stdout);
                        error = YES;
                        goto end;
                        }
   
                   octalstr[0] = cc & 0xff;
                   octalstr[1] = capstr[ capindex++];
                   octalstr[2] = capstr[ capindex++];
                   octalstr[3] = '\0';
                   sscanf( octalstr, "%o", &cc);
                   /* printf( "octal = %s, cc = %d\n", octalstr, cc); */
                   }

                 else {
                     ptermseq[ index][ seqindex++] = '\\';
                     }
                
                } /* end of switch statement */

            }  /* end of if statement '\\' */

          else if ( cc == '^') {

              if ( capindex >= caplength ) {
                printf( "Unexpected end of cap string after '^' (%d)",
                        linectr);
                fflush( stdout);
                error = YES;
                goto end;
                }

               cc = capstr[ capindex++] & 0xff;
               if ( cc > 95 || cc < 64)  {
                  cc = '^';
                  --capindex;
                  goto setterm;
                  }

               cc -= 64;

               } /* end of '^' if statement */

        setterm:
           ptermseq[ index][ seqindex++] = cc & 0xff;

           }   /* end of convert capstr loop  */

     ptermseq[ index][ seqindex] = '\0';

     /* printf( "readcap %d: sequence (%s)", index, ptermseq[ index]); */

     }   /* end of outer for loop  */


end:

   if ( error == YES) {
        printf("\r\n\r\nPress RETURN key to continue...");
        termgetline(  line, sizeof line, stdin);
        printf("\r\n\r\n\r\n");
        }

}


int fxaddterm(filename)
char *filename;
{
    FILE      *fp1;
    char line[133];
    char *strchr();

    if ((fp1 = fopen(filename, "a")) == NULL) {
        printf("can't open %s (%d)\n", filename, errno);
        return(-1);
    }
    printf("\nEnter terminal name to add (q=quit)>");
    termgetline(line, sizeof( line), stdin);
    if (*line == 'q' || *line == 'Q')
       exit(0);
    if (strchr(line, ':') == (char *)0)
       strcat(line, "  :  user added");
    strcat(line, "\n");
    fputs(line, fp1);

    fclose(fp1);
    return(0);
}
