#include <stdio.h>
#include <string.h>

extern char *optarg;
extern int optind;

char version[] = "2.0" ;

void help()
{
    printf( "\n" ) ;
    printf( "Command line format:\n" ) ;
    printf( "\ttoanita [options] unixfilename [pcfilename]\n" ) ;
    printf( "\n" ) ;
    printf( "Options:\n" ) ;
    printf( "\t-a 1           enable acks, defaults to disabled\n" ) ;
    printf( "\t-b size        block size, defaults to 2048\n" ) ;
    printf( "\t-H             when using AniTa in HP emulation mode.\n" ) ;
    printf( "\t-t             text mode (convert lines ending with LF to CR LF).\n" ) ;
    printf( "\t-h             show this help information.\n" ) ;
    printf( "\n" ) ;
    printf( "Examples:\n" ) ;
    printf( "\ttoanita helpicon.bmp\n" ) ;
    printf( "\ttoanita -t -b 2048 -a 1  file.txt file.text\n" ) ;
    printf( "\n" ) ;
}


int main(argc,argv)
int argc;
char *argv[];
{
int i;
int ch;
int lastch;
int buffer_size = 2048;
char *input_file = 0 ;
char *output_file = 0 ;
int do_ack = 0 ;
int text_mode = 0 ;
FILE *handle ;
#define max_buffer_size (16*1024)
unsigned char buffer[ max_buffer_size*2 ] ;
#define max_ack_size 32
unsigned char ack[ max_ack_size ] ;
long numbytes, file_sz ;
char csi_char = '[' ;

    while (( i = getopt( argc, argv, "ta:b:Hhx" )) != -1 ) switch( i )
        {
        case 'a':
            sscanf (optarg, "%u", &i);
            do_ack = i ;
            break ;
        case 't':
            text_mode = 1;
            break ;
        case 'b':
            sscanf (optarg, "%u", &i);
            buffer_size = i ;
            if ( buffer_size > max_buffer_size )
                {
                fprintf( stderr, "Buffer size is too large!\n" ) ;
	        exit( 3 ) ;
                }
            break ;
        case 'H':
            csi_char = '%' ;
            break ;
        case 'h':
            printf( "File Transfer To AniTa - version %s\n", version ) ;
            help() ;
            exit( 0 ) ;
        case 'x':
            exit( 0 ) ;
        }

    if (optind < argc )
        {
        input_file = argv[optind];
        }
    else
        {
        fprintf(stderr,"Error: You must specify a unix filename as input!\n");
        help() ;
	exit( 1 ) ;
        }

    if (optind+1 < argc )
        {
        output_file = argv[optind+1];
        }
    else
        {
        output_file = strrchr( input_file, '/' ) ;
	if ( output_file != 0 ) output_file++ ;
	if ( (output_file == 0) || (*output_file == 0) )
		output_file = input_file ;
        }

    handle = fopen( input_file, "r" ) ;
    if ( handle == 0 )
        {
        fprintf( stderr, "Could not open input file %s !\n", input_file ) ;
	exit( 2 ) ;
        }
    if ( text_mode )
	    {
		lastch = 0 ;
		while ((ch = getc( handle )) != EOF )
			{
			file_sz++;
			if ( ( ch == '\n' ) && ( lastch != '\r' ) ) file_sz++;
			lastch = ch ;
			}
		}
	else
		{
	    fseek( handle, 0L, 2 ) ;
	    file_sz = ftell( handle ) ;
		}
    rewind( handle ) ;

/* GWM 7/5/05
    printf( "%s", output_file ) ;
*/
    printf( "\033%c13;%ldy%s\033\134", csi_char, file_sz, output_file ) ;

    system( "stty -echo" ) ;

    printf( "\033%c34;-1;1y", csi_char ) ;
    fgets( ack, max_ack_size, stdin ) ;
    if ( strncmp( ack, "OK", 2 ) != 0 )
        {
    	system( "stty echo" ) ;
    	printf( "Could not open output file: %s\n", ack ) ;
    	exit( -1 ) ;
        }

    system( "stty -opost" ) ;

		if( text_mode )
			{
			lastch = 0 ;
			numbytes = 0L ;
		    while ((ch = getc( handle )) != EOF )
		    	{
				if ( ( ch == '\n' ) && ( lastch != '\r' ) )
					buffer[numbytes++] = '\r' ;
				buffer[numbytes++] = (char)ch ;
				if (numbytes >= buffer_size)
					{
			    	printf( "\033%c34;%d;%dy", csi_char, numbytes, do_ack ) ;
			        fwrite( buffer, 1, numbytes, stdout ) ;
			        if ( do_ack ) fgets( ack, max_ack_size, stdin ) ;
			        numbytes = 0 ;
					}
				lastch = ch;
				}
			if (numbytes > 0)
				{
		    	printf( "\033%c34;%d;%dy", csi_char, numbytes, do_ack ) ;
		        fwrite( buffer, 1, numbytes, stdout ) ;
		        if ( do_ack ) fgets( ack, max_ack_size, stdin ) ;
				}
			}
		else /* binary mode */
			{
		    while ((numbytes = fread( buffer, 1, buffer_size, handle )) > 0)
		        {
		    	printf( "\033%c34;%d;%dy", csi_char, numbytes, do_ack ) ;
		        fwrite( buffer, 1, numbytes, stdout ) ;
		        if ( do_ack ) fgets( ack, max_ack_size, stdin ) ;
			}
		}
    printf( "\033%c34;0;%dy", csi_char, do_ack ) ;
    if ( do_ack ) fgets( ack, max_ack_size, stdin ) ;

    fclose( handle ) ;

    system( "stty opost" ) ;
    system( "stty echo" ) ;
    exit( 0 ) ;
}
